@@ -9,22 +9,13 @@ except ImportError:
 from django.utils.decorators import method_decorator
 
 import gevent
-from gevent import event
-from gevent import queue
-from gevent import sleep
 
+class TimeoutPolling(Exception):
+    pass
 
 class BaseLongPollingView(View):
     timeout = 30
 
-    def __init__(self, *args, **kwargs):
-        super(BaseLongPollingView, self).__init__(*args, **kwargs)
-        self.greenlets = []
-        self.stream = queue.Queue()
-        self.response_completed = event.Event()
-        self.spawn(self.cleanup_greenlets)
-        self.spawn(self.timeout_ticker, self.timeout)
-
     @method_decorator(csrf_exempt)
     def dispatch(self, request, *args, **kwargs):
         if request.method.lower() in self.http_method_names:
@@ -48,28 +39,15 @@ class BaseLongPollingView(View):
         return self.iterator()
 
     def _iterator(self, handler):
-        self.spawn(handler, self.request)
-        for chunk in self.stream:
-            yield chunk
-
-    def cleanup_greenlets(self):
-        self.response_completed.wait()
-        for g in reversed(self.greenlets):
-            g.kill()
-
-    def spawn(self, fn, *args, **kwargs):
-        self.greenlets.append(gevent.spawn(fn, *args, **kwargs))
-
-    def close_connection(self):
-        self.response_completed.set()
-        self.write(StopIteration)
-
-    def timeout_ticker(self, timeout):
-        sleep(timeout)
-        self.close_connection()
-
-    def write(self, data):
-        self.stream.put(data)
+        timeout = gevent.Timeout(10, TimeoutPolling)
+        timeout.start()
+        try:
+            for chunk in handler(self.request):
+                yield chunk
+        except TimeoutPolling:
+            pass
+        finally:
+            timeout.cancel()
 
     def iterator(self):
         raise NotImplementedError()
